# MiSaSiM Simulator
# Written by Linda and Scott Wills
# (c) 2022 Scott & Linda Wills
# All rights reserved.

from __future__ import print_function
from builtins import input
from builtins import object

from tkinter        import *
from tkinter.messagebox   import *
from urllib.request import urlopen, urlretrieve
from os             import listdir, mkdir, remove, rename
from sys            import version

MiSaSiMURL = 'http://lindawills.ece.gatech.edu/misasim/'
Version = 3.07
ReqPython = ["3.11"]

class Manager(object) :
    """ This class supports installation, upgrading and maintanance, and running
    of MiSaSiM. """
    def __init__(Self, Version, URL) :
        Self.Run = False
        PyVersion = Self.Get_Python_Version()
        WebVersion = Self.Get_Version(URL)
        BinDir, AsmDir = Self.Get_Dirs()
        if PyVersion not in ReqPython :
            print("WARNING: MiSaSiM needs Python %s, %s, or %s; Python %s installed" % (ReqPython[0], ReqPython[1], ReqPython[2], PyVersion))
        elif WebVersion :
            if BinDir == "../bin/" and Version >= WebVersion and PyVersion in ReqPython :
                Self.Run = True
            elif not BinDir :
                print("installing MiSaSiM ...")
                BinDir, AsmDir = Self.Create_Dirs()
                Self.Install(URL, PyVersion, BinDir, AsmDir)
                remove("./Misasim.py")
            elif Version != WebVersion :
                print("upgrading MiSaSiM to version %.2f ..." % (WebVersion))
                if not AsmDir :
                    print("creating ../asm ...")
                    mkdir("../asm")
                    AsmDir = "../asm/"
                Self.Install(URL, PyVersion, BinDir, AsmDir)
        else :
            print("cannot access MiSaSiM website")
            if BinDir == "../bin" :
                "running current version"
                Self.Run = True
            else :
                "check internet access; if okay, the website must be down ... sorry"

    def Get_Version (Self, URL) :
        """ This routine attempts to download the web version number from
        the web. If it fails, the web version number is set to None. """
        try :
            return float(urlopen(URL + 'version.txt').read())
        except IOError :
            return None

    def Get_Python_Version(Self) :
        """ This routine returns the pcurrent executing python version and first
        subversion as a text string (e.g., "2.6"). """
        v = version.split('.')
        return (v[0] + '.' + v[1])

    def Get_Dirs(Self) :
        """ This routine returns the bin and asm directories. If they don't exist,
        None is returned. """
        Items = listdir("..")
        BinDir = AsmDir = None
        if 'bin' in Items :
            BinDir = '../bin/'
        if 'asm' in Items :
            AsmDir = '../asm/'
        return (BinDir, AsmDir)

    def Create_Dirs(Self) :
        """ This routine creates the bin and asm directories relative to the
        location of this function. """
        Items = listdir('.')
        if "Misasim" not in Items :
            print("creating ./Misasim ...")
            mkdir("./Misasim")
        Items = listdir('./Misasim')
        if "bin" not in Items :
            print("creating ./Misasim/bin ...")
            mkdir("./Misasim/bin")
        if "asm" not in Items :
            print("creating ./Misasim/asm ...")
            mkdir("./Misasim/asm")
        return ("Misasim/bin/", "Misasim/asm/")

    def Install(Self, URL, PyVersion, BinDir, AsmDir) :
        """ This routine populates the bin and asm sites with files from the MiSaSiM
        website. If files already exist, they are renamed. """
        BinItems = listdir(BinDir)
        AsmItems = listdir(AsmDir)
        encoding = 'utf-8'
        for FileName in urlopen(URL + 'manifest.txt') :
            # urlopen returns HTTPResponse object that contains byte encoding of
            # filenames each ending with "\n".  We need to encode as utf-8 and strip off the \n.
            FileName = str(FileName[:-1], encoding)
            if FileName[-4:] == ".asm" :
                Self.Upgrade(URL, FileName, AsmDir, AsmItems)
            elif FileName[-4:] == ".pyc" :
                Self.Upgrade(URL + PyVersion + '/', FileName, BinDir, BinItems)
            elif FileName != "" :
                Self.Upgrade(URL, FileName, BinDir, BinItems)
        print("installation complete")

    def Upgrade(Self, URL, FileName, Dir, ExistingItems) :
        """ This routine upgrades a filename from the web. If it already exists locally,
        it is renamed. """
        print("downloading %s into %s" % (FileName, Dir))
        if FileName in ExistingItems :
            OldFileName = FileName + '~'
            if OldFileName in ExistingItems :
                remove(Dir + OldFileName)
            rename(Dir + FileName, Dir + OldFileName)
        urlretrieve(URL + FileName, Dir + FileName)

if __name__ == '__main__':
    Mgr = Manager(Version, MiSaSiMURL)
    if Mgr.Run :
        from UserInterface import *
        Main = MiSaSiM(Version)
        Main.mainloop()
    else :
        input('press ENTER to continue ...')

